#ifndef KPROPERTY_CONFIG_H
#define KPROPERTY_CONFIG_H

/* config-kproperty.h. Generated by cmake from config-kproperty.h.cmake */

/*! @file config-kproperty.h
    Global KProperty configuration (build time)
*/

//! @def KPROPERTY_GIT_SHA1_STRING
//! @brief Indicates the git sha1 commit which was used for compilation of KProperty
/* #undef KPROPERTY_GIT_SHA1_STRING */

//! @def KPROPERTY_GIT_BRANCH_STRING
//! @brief Indicates the git branch which was used for compilation of KProperty
/* #undef KPROPERTY_GIT_BRANCH_STRING */

//! @def BIN_INSTALL_DIR
//! @brief The subdirectory relative to the install prefix for executables.
#define BIN_INSTALL_DIR "bin"

//! @def KPROPERTY_TESTING_EXPORT
//! @brief Export symbols for testing
#ifdef BUILD_TESTING
#  define KPROPERTY_TESTING_EXPORT KPROPERTY_EXPORT
#else
#  define KPROPERTY_TESTING_EXPORT
#endif

//! @def KPROPERTY_UNFINISHED
//! @brief Defined if unfinished features of KProperty are enabled
/* #undef KPROPERTY_UNFINISHED */

//! @def KPROPERTY_WIDGETS
//! @brief Defined if QtWidgets components of KProperty are enabled
#define KPROPERTY_WIDGETS

//! @def KPROPERTYCORE_BASE_NAME
//! @brief Base name for the KPropertyCore library, based on major stable version.
//!        Useful for co-installability.
#define KPROPERTYCORE_BASE_NAME "KPropertyCore3"

//! @def KPROPERTYCORE_BASE_NAME_LOWER
//! @brief Like KPROPERTYCORE_BASE_NAME but lowercase.
#define KPROPERTYCORE_BASE_NAME_LOWER "kpropertycore3"

//! @def KPROPERTYWIDGETS_BASE_NAME
//! @brief Base name for the KPropertyCore library, based on major stable version.
//!        Useful for co-installability.
#define KPROPERTYWIDGETS_BASE_NAME "KPropertyWidgets3"

//! @def KPROPERTYWIDGETS_BASE_NAME_LOWER
//! @brief Like KPROPERTYWIDGETS_BASE_NAME but lowercase.
#define KPROPERTYWIDGETS_BASE_NAME_LOWER "kpropertywidgets3"

//! @def KPROPERTY_STABLE_VERSION_MAJOR
//! @brief Extra version info, stable major, e.g. 3 for 3.1.0 Alpha (3.0.90)
#define KPROPERTY_STABLE_VERSION_MAJOR 3

//! @def KPROPERTY_STABLE_VERSION_MINOR
//! @brief Extra version info, stable minor, e.g. 1 for 3.1.0 Alpha (3.0.90)
#define KPROPERTY_STABLE_VERSION_MINOR 2

//! @def KPROPERTY_STABLE_VERSION_PATCH
//! @brief Extra version info, stable patch, e.g. 0 for 3.1.0 Alpha (3.0.90)
#define KPROPERTY_STABLE_VERSION_PATCH 0

//! @def KPROPERTY_KF
//! @brief Defined if components depending on KDE Frameworks are enabled
#define KPROPERTY_KF

#endif
